import time

import pandas as pd
import requests

from pyplattsapi import plattsapicore

# https://developer.platts.com/servicecatalog#/WorldOilSupply(Beta)/v2/

api_name = "WORLD OIL SUPPLY"
production_api = f"{plattsapicore.api_url}/wos/v2/production"


def get_production_api_call(
    filter: str, field: str, groupBy: str, page: int = 1, scenarioTermId: int = 2
):
    params = {
        "filter": filter,
        "scenarioTermId": scenarioTermId,
        "field": field,
        "pageSize": 1000,
        "groupBy": groupBy,
        "page": page,
    }
    response = requests.get(
        url=production_api, headers=plattsapicore.build_header(api_name), params=params
    )
    if response.status_code == 200:
        return response.json()


def get_production(filter, field, groupBy):
    r = get_production_api_call(filter, field, groupBy)
    max_page = int(r["metadata"]["count"] / 1000)

    res = pd.concat([pd.Series(x) for x in r["results"]], axis=1)
    for page in range(2, max_page + 2):
        time.sleep(0.55)
        r = get_production_api_call(filter, field, groupBy, page=page)
        d = pd.concat([pd.Series(x) for x in r["results"]], axis=1)
        res = pd.concat([d, res], axis=1)

    res = res.T
    res.index = res.apply(lambda x: pd.to_datetime(f"{x.year}-{x.month}-1"), 1)
    res.index.name = "date"
    return res



import os
from functools import lru_cache
from cachetools.func import ttl_cache

import requests

api_url = "https://api.platts.com"
auth_url = f"{api_url}/auth/api"


@lru_cache(maxsize=20)
def extract_credential(key: str):
    credentials = os.getenv("PLATTS_API_CREDENTIALS")
    tokens = credentials.split(";")
    token = [x for x in tokens if x.startswith(key)]
    if len(token) > 0:
        s = token[0].split("=")
        return s[1]


def get_access_token(api_dataset):
    """
    Given an api_dataset (eg OIL INVENTORY), get an access token from the Auth API
    :param api_dataset:
    :return:
    """
    data = {
        "username": extract_credential("username"),
        "password": extract_credential("password"),
    }
    headers = {"appkey": extract_credential(api_dataset)}
    token_request_sup = requests.post(auth_url, headers=headers, data=data)
    req_dic_sup = token_request_sup.json()
    access_token_sup = req_dic_sup["access_token"]
    return access_token_sup


@ttl_cache(ttl=10 * 60)
def build_header(api_dataset):
    header = {
        "accept": "application/json",
        "appkey": extract_credential(api_dataset),
        "Authorization": f"Bearer {get_access_token(api_dataset)}",
    }
    return header



import time

import pandas as pd
import requests

from plattsapi.plattsapicore import Headers_ref, Headers_sup


def getCountryYearlyRunsTimeSeries(country: str):
    Historical_data_URL = f"https://api.platts.com/odata/refinery-data/v2/Runs?$select=*&$expand=*&pageSize=1000&$filter=Refinery/Country/Name eq '{country}'"
    df5 = pd.DataFrame()
    while Historical_data_URL != "NaN":
        time.sleep(1)  # api can only accept 2 requests per second and 5000 per day
        data_request = requests.get(url=f"{Historical_data_URL}", headers=Headers_ref)
        data = data_request.json()
        df2 = pd.json_normalize(data).reset_index(drop=True)
        x = df2["value"].iloc[0]
        df3 = pd.json_normalize(x).reset_index(drop=True)
        df3 = df3.drop_duplicates()
        df4 = df3[["Year", "Refinery.Name", "Mbcd", "Mmtcd", "Mmtcy"]]
        df4 = df4.groupby(["Year", "Refinery.Name"]).sum()
        df5 = df5.append(df4, ignore_index=False)
        try:
            Historical_data_URL = df2[f"@odata.nextLink"].iloc[0]
        except:
            Historical_data_URL = "NaN"
            continue
    df5 = df5.reset_index()
    df5.columns = ["Year", "Refinery", "Mbdc", "Mmtcd", "Mmtcy"]
    df7 = df5.groupby(["Year"]).sum()
    df7 = df7.reset_index()
    return df7


def getMarginsbyType(type: str):
    Historical_data_URL = f"https://api.platts.com/odata/refinery-data/v2/Margins?&pageSize=1000&$expand=*"
    df5 = pd.DataFrame()
    while Historical_data_URL != "NaN":
        time.sleep(1)  # api can only accept 2 requests per second and 5000 per day
        data_request = requests.get(url=f"{Historical_data_URL}", headers=Headers_ref)
        data = data_request.json()
        df2 = pd.json_normalize(data).reset_index(drop=True)
        x = df2["value"].iloc[0]
        df3 = pd.json_normalize(x).reset_index(drop=True)
        df3 = df3.drop_duplicates()
        df5 = df5.append(df3, ignore_index=False)
        try:
            Historical_data_URL = df2[f"@odata.nextLink"].iloc[0]
        except:
            Historical_data_URL = "NaN"
            continue
    df5 = df5[df5["MarginType.Name"] == type]
    df5 = df5.reset_index()
    return df5


def getCountryCapacityChangesTimeSeries(country: str):
    Historical_data_URL = f"https://api.platts.com/odata/refinery-data/v2/capacity?$select=*&$expand=*&pageSize=1000&$filter=Refinery/Country/Name eq '{country}'"
    df5 = pd.DataFrame()
    while Historical_data_URL != "NaN":
        time.sleep(1)  # api can only accept 2 requests per second and 5000 per day
        data_request = requests.get(url=f"{Historical_data_URL}", headers=Headers_sup)
        data = data_request.json()
        df2 = pd.json_normalize(data).reset_index(drop=True)
        x = df2["value"].iloc[0]
        df3 = pd.json_normalize(x).reset_index(drop=True)
        df3 = df3.drop_duplicates()
        df3["Date"] = df3[["Year", "Quarter"]].apply(
            lambda row: "-Q".join(row.values.astype(str)), axis=1
        )
        df4 = df3[["Refinery.Name", "Mbcd", "Mmtcd", "Mmtcy", "Date"]]
        df4 = df4.groupby(["Date", "Refinery.Name"]).sum()
        df4 = df4.reset_index()
        df4["Date"] = pd.to_datetime(df4["Date"])
        df5 = df5.append(df4, ignore_index=False)
        try:
            Historical_data_URL = df2[f"@odata.nextLink"].iloc[0]
        except:
            Historical_data_URL = "NaN"
            continue
    df4.columns = ["Date", "Refinery", "Mbdc", "Mmtcd", "Mmtcy"]
    df5 = df4.groupby(["Date"]).sum()
    df6 = df5.reset_index()
    return df6
